/*
 * File-specific pager functionality.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/mapped_file_object_interface.h>
#include <fsserver/pager.h>
#include <fsserver/file_provider.h>



/* A pager abstraction for a file. */

class FilePager : public Pager, public MappedFileObject
{
protected:
    FileProvider *_provider;
    flags_t _flags;

    /* Notification endpoint for event subscription. */

    unsigned int _endpoint;

    /* Resize flag for notification. */

    bool _resized = false;

    /* Helper methods. */

    bool copy_on_write();

public:
    fileid_t fileid;

    explicit FilePager(fileid_t fileid, FileProvider *provider, flags_t flags);

    virtual void close();

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<MappedFileObject *>(this); }

    /* File methods. */

    virtual long flush(offset_t populated_size, offset_t *size);

    virtual long resize(offset_t *size);

    /* Pager and mapped file methods. */

    virtual long map(offset_t offset, map_address_t hot_spot, map_flags_t flags,
                     l4_snd_fpage_t *region);

    virtual long mmap(offset_t position, offset_t length,
                      offset_t start_visible, offset_t end_visible,
                      offset_t *start_pos, offset_t *end_pos, offset_t *size);

    /* Notification methods. */

    virtual long subscribe(l4_cap_idx_t notifier, notify_flags_t flags);

    virtual long unsubscribe();
};

// vim: tabstop=4 expandtab shiftwidth=4
